/****************************************************************************
 *	pcm_ioc.c: УãͲϢ
 *
 *			Date. 1994/07/20 --
 *				Programed by Nobuyuki Maruichi
 *				Adviced by S.Kouno
 *
 *	from
 *		PCM player T-E-S-T program  Copyright (C) 1993 S.Kouno
 *		PCM recording T-E-S-T program  Copyright (C) 1993 S.Kouno
 ****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>

#include "snd.h"

/***************************************************************************
 *	make_sndHeader: FM TOWNS*.SNDɷեΥإå
 *	
 *		void make_sndHeader(const char *name, 
 *								long wsize, struct snd_head *head);
 *	
 *		const char *name;
 *		long wsize;
 *		struct snd_head *head;
 *	
 *		
 ***************************************************************************/
void make_sndHeader(const char *name, long wsize, struct snd_head *head)
{
	int length;

	memset(head, 0, sizeof(*head));
	length = strlen(name);
	if(length > sizeof(head->name)) {
		length = sizeof(head->name);
	}
	memcpy(head->name, name, length);
	head->length = wsize;
	head->rate = SND_RATE * 0x62 / 1000;  /* 19.2 kHz */
	head->note = 0x3c;
}

/***************************************************************************
 *	encode2sndFormat: FM TOWNS*.SNDɷեѴ
 *	
 *		void encode2sndFormat(long wsize, unsigned char *pcmData_ptr);
 *	
 *		long wsize;
 *		unsigned char *pcmData_ptr;
 *	
 *		
 ***************************************************************************/
void encode2sndFormat(long wsize, unsigned char *pcmData_ptr)
{
	unsigned char *ptr;
	unsigned int c;
	int size;
	
	for (size=0,ptr=pcmData_ptr;size<wsize;size++,ptr++) {
		c = (unsigned int)(*ptr);
		if (c<1) c=1;
		if (c<0x80) {
			c = 0x80 - c;
		}
		if (c>0xfe) c=0xfe;
		*ptr = c;
	}
}

#define voltage(c)	(((c) & 0x80)?((c) & 0x7f):-(c))

/***************************************************************************
 *	saveSndData: 
 *	
 *		int saveSndData(const char *filename, 
 *						const struct *snd_head head, unsigned char *buff);
 *	
 *		const char *filename;
 *		const struct *snd_head head;
 *		unsigned char *buff;
 *	֤
 *		
 *	
 *		
 ***************************************************************************/
int saveSndData(const char *filename, 
				const struct snd_head *head, unsigned char *buff)
{
	FILE *fp;

	/*
	 * write all 
	 */
	fp = fopen(filename, "w");
	if (fp==NULL) {
		fprintf(stderr,"%s: Can't open SND file '%s'\n", 
				__FILE__, filename);
		return(1);
	}
	if (fwrite((char*)head, sizeof(*head), 1, fp)!=1) {
		fprintf(stderr,"%s: Can't write SND file '%s'\n", 
				__FILE__, filename);
		return(1);
	}
	if (fwrite((char*)buff, head->length, 1, fp)!=1) {
		fprintf(stderr,"%s: Can't write SND file '%s'\n", 
				__FILE__, filename);
		return(1);
	}
	fclose(fp);
	
	return(0);
}

/***************************************************************************
 *	: 
 *	
 *		;
 *	
 *		;
 *	֤
 *		
 *	
 *		
 ***************************************************************************/

/***************************************************************************
 *	say_sample: say ̾Σäδؿ
 *	
 *		int say_sample(const char *snd_file, int spos, int epos);
 *	
 *		const char *snd_file;	SND եΥե̾
 *		int spos;				ϰ֡ʥǥեȡ
 *		int epos;				λ֡ʥǥեȡ0x7fffffff
 *	֤
 *		
 *	
 *		
 ***************************************************************************/
int say_sample(const char *snd_file, int spos, int epos)
{

	FILE *fd1, *fd2;
	char buff[BUFSIZ];
	char *ptr;
	struct snd_head head;
	int rsize;
	int wsize;

	fd1 = fopen(snd_file, "r");
	if (fd1==NULL) {
		return(EOF);
	}
	/*
	 * PCM player
	 */
	fd2 = fopen("/dev/pcm","w");
	if (fd2==NULL) {
		fprintf(stderr,"%s: Can't open PCM driver\n", "say_sample()");
		fclose(fd1);
		return(EOF);
	}

	fwrite(&head.rate,sizeof(head.rate),1,fd2); /* put sampling rate */


	/*
	 * write PCM data between spos and epos
	 */
	fseek(fd1,sizeof(head)+spos,0);
	while ((epos>0)&&((rsize=fread(buff,1,BUFSIZ,fd1))>0)) {
		ptr = buff;
		while (rsize>0) {
			wsize=fwrite(ptr,1,rsize,fd2);
			rsize -= wsize;
			ptr += wsize;
			epos -= wsize;
		}
	}

	fclose(fd2);
	fclose(fd1);

	return(0);
}

